@@ -18,6 +18,7 @@ except ImportError:
     load_source = imp.load_source
 
 from os.path import isabs
+from pelican.utils import posix_join
 
 from pelican.log import LimitFilter
 
@@ -41,11 +42,11 @@ DEFAULT_CONFIG = {
     'STATIC_EXCLUDE_SOURCES': True,
     'THEME_STATIC_DIR': 'theme',
     'THEME_STATIC_PATHS': ['static', ],
-    'FEED_ALL_ATOM': os.path.join('feeds', 'all.atom.xml'),
-    'CATEGORY_FEED_ATOM': os.path.join('feeds', '%s.atom.xml'),
-    'AUTHOR_FEED_ATOM': os.path.join('feeds', '%s.atom.xml'),
-    'AUTHOR_FEED_RSS': os.path.join('feeds', '%s.rss.xml'),
-    'TRANSLATION_FEED_ATOM': os.path.join('feeds', 'all-%s.atom.xml'),
+    'FEED_ALL_ATOM': posix_join('feeds', 'all.atom.xml'),
+    'CATEGORY_FEED_ATOM': posix_join('feeds', '%s.atom.xml'),
+    'AUTHOR_FEED_ATOM': posix_join('feeds', '%s.atom.xml'),
+    'AUTHOR_FEED_RSS': posix_join('feeds', '%s.rss.xml'),
+    'TRANSLATION_FEED_ATOM': posix_join('feeds', 'all-%s.atom.xml'),
     'FEED_MAX_ITEMS': '',
     'SITEURL': '',
     'SITENAME': 'A Pelican Blog',
@@ -61,32 +62,29 @@ DEFAULT_CONFIG = {
     'NEWEST_FIRST_ARCHIVES': True,
     'REVERSE_CATEGORY_ORDER': False,
     'DELETE_OUTPUT_DIRECTORY': False,
-    'OUTPUT_RETENTION': (),
+    'OUTPUT_RETENTION': [],
     'ARTICLE_URL': '{slug}.html',
     'ARTICLE_SAVE_AS': '{slug}.html',
-    'ARTICLE_ORDER_BY': 'slug',
+    'ARTICLE_ORDER_BY': 'reversed-date',
     'ARTICLE_LANG_URL': '{slug}-{lang}.html',
     'ARTICLE_LANG_SAVE_AS': '{slug}-{lang}.html',
     'DRAFT_URL': 'drafts/{slug}.html',
-    'DRAFT_SAVE_AS': os.path.join('drafts', '{slug}.html'),
+    'DRAFT_SAVE_AS': posix_join('drafts', '{slug}.html'),
     'DRAFT_LANG_URL': 'drafts/{slug}-{lang}.html',
-    'DRAFT_LANG_SAVE_AS': os.path.join('drafts', '{slug}-{lang}.html'),
+    'DRAFT_LANG_SAVE_AS': posix_join('drafts', '{slug}-{lang}.html'),
     'PAGE_URL': 'pages/{slug}.html',
-    'PAGE_SAVE_AS': os.path.join('pages', '{slug}.html'),
+    'PAGE_SAVE_AS': posix_join('pages', '{slug}.html'),
     'PAGE_ORDER_BY': 'basename',
     'PAGE_LANG_URL': 'pages/{slug}-{lang}.html',
-    'PAGE_LANG_SAVE_AS': os.path.join('pages', '{slug}-{lang}.html'),
+    'PAGE_LANG_SAVE_AS': posix_join('pages', '{slug}-{lang}.html'),
     'STATIC_URL': '{path}',
     'STATIC_SAVE_AS': '{path}',
-    'PDF_GENERATOR': False,
-    'PDF_STYLE_PATH': '',
-    'PDF_STYLE': 'twelvepoint',
     'CATEGORY_URL': 'category/{slug}.html',
-    'CATEGORY_SAVE_AS': os.path.join('category', '{slug}.html'),
+    'CATEGORY_SAVE_AS': posix_join('category', '{slug}.html'),
     'TAG_URL': 'tag/{slug}.html',
-    'TAG_SAVE_AS': os.path.join('tag', '{slug}.html'),
+    'TAG_SAVE_AS': posix_join('tag', '{slug}.html'),
     'AUTHOR_URL': 'author/{slug}.html',
-    'AUTHOR_SAVE_AS': os.path.join('author', '{slug}.html'),
+    'AUTHOR_SAVE_AS': posix_join('author', '{slug}.html'),
     'PAGINATION_PATTERNS': [
         (0, '{name}{number}{extension}', '{name}{number}{extension}'),
     ],
@@ -95,11 +93,9 @@ DEFAULT_CONFIG = {
     'DAY_ARCHIVE_SAVE_AS': '',
     'RELATIVE_URLS': False,
     'DEFAULT_LANG': 'en',
-    'TAG_CLOUD_STEPS': 4,
-    'TAG_CLOUD_MAX_ITEMS': 100,
-    'DIRECT_TEMPLATES': ('index', 'tags', 'categories', 'authors', 'archives'),
+    'DIRECT_TEMPLATES': ['index', 'tags', 'categories', 'authors', 'archives'],
     'EXTRA_TEMPLATES_PATHS': [],
-    'PAGINATED_DIRECT_TEMPLATES': ('index', ),
+    'PAGINATED_DIRECT_TEMPLATES': ['index'],
     'PELICAN_CLASS': 'pelican.Pelican',
     'DEFAULT_DATE_FORMAT': '%a %d %B %Y',
     'DATE_FORMATS': {},
@@ -110,7 +106,7 @@ DEFAULT_CONFIG = {
     'LOCALE': [''],  # defaults to user locale
     'DEFAULT_PAGINATION': False,
     'DEFAULT_ORPHANS': 0,
-    'DEFAULT_METADATA': (),
+    'DEFAULT_METADATA': {},
     'FILENAME_METADATA': '(?P<date>\d{4}-\d{2}-\d{2}).*',
     'PATH_METADATA': '',
     'EXTRA_PATH_METADATA': {},
@@ -127,14 +123,14 @@ DEFAULT_CONFIG = {
     'SLUG_SUBSTITUTIONS': (),
     'INTRASITE_LINK_REGEX': '[{|](?P<what>.*?)[|}]',
     'SLUGIFY_SOURCE': 'title',
-    'CACHE_CONTENT': True,
+    'CACHE_CONTENT': False,
     'CONTENT_CACHING_LAYER': 'reader',
     'CACHE_PATH': 'cache',
     'GZIP_CACHE': True,
     'CHECK_MODIFIED_METHOD': 'mtime',
-    'LOAD_CONTENT_CACHE': True,
-    'AUTORELOAD_IGNORE_CACHE': False,
+    'LOAD_CONTENT_CACHE': False,
     'WRITE_SELECTED': [],
+    'FORMATTED_FIELDS': ['summary'],
     }
 
 PYGMENTS_RST_OPTIONS = None
