@@ -26,7 +26,7 @@ SOCIAL = (('twitter', 'http://twitter.com/ametaireau'),
           ('github', 'http://github.com/ametaireau'),)
 
 # global metadata to all the contents
-DEFAULT_METADATA = (('yeah', 'it is'),)
+DEFAULT_METADATA = {'yeah': 'it is'}
 
 # path-specific metadata
 EXTRA_PATH_METADATA = {
@@ -39,6 +39,9 @@ STATIC_PATHS = [
     'extra/robots.txt',
     ]
 
+FORMATTED_FIELDS = ['summary', 'custom_formatted_field']
+
 # foobar will not be used, because it's not in caps. All configuration keys
 # have to be in caps
 foobar = "barbaz"
+
