@@ -3,6 +3,7 @@ from __future__ import unicode_literals, print_function
 import copy
 import os
 import locale
+from sys import platform
 from os.path import dirname, abspath, join
 
 from pelican.settings import (read_settings, configure_settings,
@@ -40,7 +41,7 @@ class TestSettingsConfiguration(unittest.TestCase):
         self.assertNotIn('foobar', self.settings)
 
     def test_read_empty_settings(self):
-        # Providing no file should return the default values.
+        # Ensure an empty settings file results in default settings.
         settings = read_settings(None)
         expected = copy.deepcopy(DEFAULT_CONFIG)
         # Added by configure settings
@@ -66,8 +67,8 @@ class TestSettingsConfiguration(unittest.TestCase):
         settings['SITENAME'] = 'Not a Pelican Blog'
         self.assertNotEqual(settings['SITENAME'], DEFAULT_CONFIG['SITENAME'])
 
-    def test_path_settings_safety(self):
-        """Don't let people setting the static path listings to strs"""
+    def test_static_path_settings_safety(self):
+        # Disallow static paths from being strings
         settings = {'STATIC_PATHS': 'foo/bar',
                 'THEME_STATIC_PATHS': 'bar/baz',
                 # These 4 settings are required to run configure_settings
@@ -83,8 +84,7 @@ class TestSettingsConfiguration(unittest.TestCase):
                 DEFAULT_CONFIG['THEME_STATIC_PATHS'])
 
     def test_configure_settings(self):
-        #Manipulations to settings should be applied correctly.
-
+        # Manipulations to settings should be applied correctly.
         settings = {
                 'SITEURL': 'http://blog.notmyidea.org/',
                 'LOCALE': '',
@@ -92,6 +92,7 @@ class TestSettingsConfiguration(unittest.TestCase):
                 'THEME': DEFAULT_THEME,
                 }
         configure_settings(settings)
+
         # SITEURL should not have a trailing slash
         self.assertEqual(settings['SITEURL'], 'http://blog.notmyidea.org')
 
@@ -102,13 +103,59 @@ class TestSettingsConfiguration(unittest.TestCase):
         configure_settings(settings)
         self.assertEqual(settings['FEED_DOMAIN'], 'http://feeds.example.com')
 
-    def test_default_encoding(self):
-        # test that the default locale is set if
-        # locale is not specified in the settings
+    def test_theme_settings_exceptions(self):
+        settings = self.settings
 
-        #reset locale to python default
+        # Check that theme lookup in "pelican/themes" functions as expected
+        settings['THEME'] = os.path.split(settings['THEME'])[1]
+        configure_settings(settings)
+        self.assertEqual(settings['THEME'], DEFAULT_THEME)
+
+        # Check that non-existent theme raises exception
+        settings['THEME'] = 'foo'
+        self.assertRaises(Exception, configure_settings, settings)
+
+    def test_deprecated_dir_setting(self):
+        settings = self.settings
+
+        settings['ARTICLE_DIR'] = 'foo'
+        settings['PAGE_DIR'] = 'bar'
+
+        configure_settings(settings)
+
+        self.assertEqual(settings['ARTICLE_PATHS'], ['foo'])
+        self.assertEqual(settings['PAGE_PATHS'], ['bar'])
+
+        with self.assertRaises(KeyError):
+            settings['ARTICLE_DIR']
+            settings['PAGE_DIR']
+
+    @unittest.skipIf(platform == 'win32', "Doesn't work on Windows")
+    def test_default_encoding(self):
+        # Test that the default locale is set if not specified in settings
+
+        # Reset locale to Python's default locale
         locale.setlocale(locale.LC_ALL, str('C'))
         self.assertEqual(self.settings['LOCALE'], DEFAULT_CONFIG['LOCALE'])
 
         configure_settings(self.settings)
         self.assertEqual(locale.getlocale(), locale.getdefaultlocale())
+
+    def test_invalid_settings_throw_exception(self):
+        # Test that the path name is valid
+
+        # test that 'PATH' is set
+        settings = {
+        }
+
+        self.assertRaises(Exception, configure_settings, settings)
+
+        # Test that 'PATH' is valid
+        settings['PATH'] = ''
+        self.assertRaises(Exception, configure_settings, settings)
+
+        # Test nonexistent THEME 
+        settings['PATH'] = os.curdir
+        settings['THEME'] = 'foo'
+
+        self.assertRaises(Exception, configure_settings, settings)
